@@ -53,12 +53,14 @@ namespace Banter
 		private Gdk.Pixbuf appPixBuf;
 		private Gnome.Program program = null;
 		private Dictionary <uint, Person> persons;
-		private Dictionary <Person, ChatWindow> chatWindows;
+		private Dictionary <string, ChatWindow> chatWindows;
 		//private Person me;
 		private PreferencesDialog preferencesDialog;
 		//VideoConversation videoConversation;
 		private Conversation conversation = null;
 		private bool initiatedConversation = false;
+		
+		private Dictionary<GroupWindow, GroupWindow> groupWindows;
 
 		// Major temporary hack
 		//private Novell.Rtc.Connection conn = null;
@@ -121,6 +123,9 @@ namespace Banter
 						Gnome.Modules.UI,
 						args);
 
+			// Call Init before anything so the people objects are in place
+			PersonStore.Instance.Init();
+			
 			actionManager = new ActionManager ();
 			actionManager.LoadInterface ();
 			SetUpGlobalActions ();
@@ -130,17 +135,18 @@ namespace Banter
 
 			application = this;
 			
-			chatWindows = new Dictionary<Person,ChatWindow> ();
+			groupWindows = new Dictionary<GroupWindow, GroupWindow> ();
+			chatWindows = new Dictionary<string,ChatWindow> ();
 
 			GLib.Idle.Add(InitializeIdle);
 		}
 	
 		private void SetupTrayIcon ()
 		{
-			Console.WriteLine ("Crating TrayIcon");
+			Console.WriteLine ("Creating TrayIcon");
 			
 			EventBox eb = new EventBox();
-			appPixBuf = GetIcon ("banter-16", 16);
+			appPixBuf = GetIcon ("banter-22", 22);
 			eb.Add(new Image (appPixBuf)); 
 			//new Image(Gtk.Stock.DialogWarning, IconSize.Menu)); // using stock icon
 
@@ -194,9 +200,35 @@ namespace Banter
 		
 		private void OnAccountManagementReadyEvent ()
 		{
-			Console.WriteLine("OneAccountManagementReady() called");
+			Logger.Debug("OnAccountManagementReady() called");
 			InitializePersons();
 			initialized = true;
+			
+			GLib.Idle.Add (OpenSavedGroupWindows);
+		}
+		
+		private bool OpenSavedGroupWindows ()
+		{
+			GroupWindow gw;
+			string[] groupWindowIds = Preferences.Get (Preferences.GroupWindows) as String[];
+			if (groupWindowIds == null) {
+				// Create a new Group Window with Everyone selected
+				gw = new GroupWindow ();
+				groupWindows [gw] = gw;
+				gw.DeleteEvent += OnGroupWindowDeleted;
+				gw.ShowAll ();
+				return false;
+			}
+			
+			foreach (string id in groupWindowIds) {
+				gw = new GroupWindow (id);
+				
+				groupWindows [gw] = gw;
+				gw.DeleteEvent += OnGroupWindowDeleted;
+				gw.ShowAll ();
+			}
+			
+			return false; // Prevent GLib.Idle from calling this method again
 		}
 
 		private void OnNewChannel (Tapioca.Connection sender, Tapioca.Channel channel)
@@ -232,16 +264,18 @@ namespace Banter
 						conversation.SetTextChannel (txtChannel);
 
 						Person peer = GetPersonFromContact (txtChannel.RemoteTarget as Contact);
-						if (chatWindows.ContainsKey (peer))
-							chatWindows[peer].Present();
-						else
-						{
+						ChatWindow cw = null;
+						
+						if (chatWindows.ContainsKey (peer.Id)) {
+							chatWindows[peer.Id].Present();
+						}
+						else {
 							try
 							{
-								ChatWindow cw = new ChatWindow ();
+								cw = new ChatWindow ();
 								cw.Conversation = conversation;
 								cw.DeleteEvent += OnChatWindowDelete;
-								chatWindows[peer] = cw;
+								chatWindows[peer.Id] = cw;
 								cw.Present();
 							}
 							catch (Exception es)
@@ -319,7 +353,9 @@ namespace Banter
 						Logger.Debug ("# Known Contacts: {0}", tapConnection.ContactList.KnownContacts.Length);
 
 						Logger.Debug ("Connection jabber requested");
-		 				foreach (Contact c in tapConnection.ContactList.SubscribedContacts)
+						
+						// Loop through and add all of the subscribed contacts
+		 				foreach (Contact c in tapConnection.ContactList.KnownContacts) //.SubscribedContacts)
 						{
 		                 	Logger.Debug (
 		                 		"Contact Retrieved\n\t{0}/{4} - {1}/{2} - {3}",
@@ -329,10 +365,19 @@ namespace Banter
 		                 		c.SubscriptionStatus, 
 		                 		c.Alias);
 
-							if(!persons.ContainsKey (c.Handle.Id))
-								persons[c.Handle.Id] = new Person (c);
+							if(!persons.ContainsKey (c.Handle.Id)) {
+								Person person = PersonStore.GetPersonByJabberId(c.Handle.Name);
+								if(person == null) {
+									person = new Person(c.Alias);
+									person.JabberId = c.Handle.Name;
+									PersonStore.AddPerson(person);
+								}
+								
+								person.Contact = c;
+								persons[c.Handle.Id] = person;
+							}
 		                }
-
+		                
 						// FIXME - For now we have all caps		                
 						tapConnection.Info.SetCapabilities (
 							ContactCapabilities.Text |
@@ -363,7 +408,8 @@ namespace Banter
 		
 		private void InitializePersons()
 		{
-			Console.WriteLine("InitializePersons called");
+			Logger.Debug("InitializePersons called");
+
 			persons = new Dictionary<uint,Person> ();
 
 			try
@@ -493,13 +539,31 @@ namespace Banter
    			{
    				// FIXME: Eventually get all these into UIManagerLayout.xml file
    				Console.WriteLine ("Right button clicked");
-      			Menu popupMenu = new Menu(); 
+      			Menu popupMenu = new Menu();
+      			
+      			if (groupWindows.Count > 0) {
+	      			// List all the opened GroupWindows here
+	      			foreach (GroupWindow groupWindow in groupWindows.Values) {
+	      				GroupWindowMenuItem item = new GroupWindowMenuItem (
+	      						groupWindow);
+	      				item.Activated += OnGroupWindowMenuItem;
+	      				popupMenu.Add (item);
+	      			}
+      			
+      				popupMenu.Add (new SeparatorMenuItem ());
+      			}
  
-      			ImageMenuItem people = new ImageMenuItem ("People");
+      			ImageMenuItem people = new ImageMenuItem (
+						Catalog.GetString ("New Group Window ..."));
       			people.Activated += OnPeople;
       			popupMenu.Add (people);
+      			
+//      			ImageMenuItem everyone = new ImageMenuItem ("Everyone");
+//      			everyone.Activated += OnEveryone;
+//      			popupMenu.Add (everyone);
 
-      			ImageMenuItem accounts = new ImageMenuItem ("Accounts");
+      			ImageMenuItem accounts = new ImageMenuItem (
+      					Catalog.GetString ("Accounts"));
       			accounts.Activated += OnAccounts;
       			popupMenu.Add (accounts);
       			
@@ -513,7 +577,8 @@ namespace Banter
       			separator = new SeparatorMenuItem ();
       			popupMenu.Add (separator);
 
-      			ImageMenuItem quit = new ImageMenuItem ("Quit");
+      			ImageMenuItem quit = new ImageMenuItem (
+      					Catalog.GetString ("Quit"));
       			quit.Activated += OnQuit;
       			popupMenu.Add (quit);
       			
@@ -533,20 +598,21 @@ namespace Banter
 		{
 			actionManager ["ShowPreferencesAction"].Activate ();
 		}
+		
+		private void OnGroupWindowMenuItem (object sender, EventArgs args)
+		{
+			GroupWindowMenuItem item = sender as GroupWindowMenuItem;
+			
+			item.GroupWindow.Present ();
+		}
 
 		private void OnPeople (object o, System.EventArgs args)
 		{
-			Console.WriteLine ("Selected People");
-
-			PersonView pv = new PersonView();
-			pv.Icon = appPixBuf;
-
-			foreach(Person p in persons.Values)
-			{
-				pv.AddPerson(p);
-			}
-
-			pv.ShowAll();
+			GroupWindow gw = new GroupWindow ();
+			
+			groupWindows [gw] = gw;
+			gw.DeleteEvent += OnGroupWindowDeleted;
+			gw.ShowAll ();
 		}
 		
 		private void OnQuit (object o, System.EventArgs args)
@@ -558,6 +624,14 @@ namespace Banter
 		{
 			Logger.Info ("OnQuitAction called - terminating application");
 
+			// Save off the GroupWindow states
+			if (groupWindows.Count > 0) {
+				Logger.Info ("Saving the state of all group windows...");
+				foreach (GroupWindow gw in groupWindows.Values) {
+					gw.SaveState ();
+				}
+			}
+			
 			/*
 			// Temporary disconnect from our single account			
 			if (conn != null && conn.Connected == true) {
@@ -624,6 +698,18 @@ namespace Banter
 				AccountManagement.Start ();
 			}
 		}
+		
+		private void OnGroupWindowDeleted (object sender, DeleteEventArgs args)
+		{
+Logger.Debug ("Application.OnGroupWindowDeleted");
+			GroupWindow gw = sender as GroupWindow;
+			gw.Hide ();
+			
+			if (groupWindows.ContainsKey (gw))
+				groupWindows.Remove (gw);
+			
+			gw.Destroy ();
+		}
 		#endregion		
 
 
@@ -743,8 +829,11 @@ namespace Banter
 		public Person GetPersonFromContact(Tapioca.Contact contact)
 		{	
 			Logger.Debug ("Application.GetPerson ({0})", contact.Uri);
-			if(persons.ContainsKey(contact.Handle.Id))
-				return persons[contact.Handle.Id];
+			if(persons.ContainsKey(contact.Handle.Id)) {
+				Person person = persons[contact.Handle.Id];
+				Logger.Debug("Found the person: {0}", person.DisplayName);
+				return person;
+			}
 			
 			throw new ApplicationException("Fixme: The dude is not found, create real exceptions");
 		}
@@ -754,15 +843,15 @@ namespace Banter
 			Console.WriteLine("Called to initiate chat with: " + person.DisplayName);
 			initiatedConversation = true;
 			
-			if(chatWindows.ContainsKey(person))
-				chatWindows[person].Present();
+			if(chatWindows.ContainsKey(person.Id))
+				chatWindows[person.Id].Present();
 			else
 			{
 				try
 				{
 					ChatWindow cw = new ChatWindow();
 					cw.DeleteEvent += OnChatWindowDelete;
-					chatWindows[person] = cw;
+					chatWindows[person.Id] = cw;
 
 					Conversation conversation = 
 						new Conversation(tapConnection, person.Contact);
@@ -816,9 +905,9 @@ namespace Banter
 			ChatWindow cw = sender as ChatWindow;
 			if (chatWindows.ContainsValue (cw))
 			{
-				Person bhck = persons[cw.Conversation.PeerContact.Handle.Id];
-				if(bhck != null)
-					chatWindows.Remove (bhck);
+				Person person = persons[cw.Conversation.PeerContact.Handle.Id];
+				if(person != null)
+					chatWindows.Remove (person.Id);
 			}
 		}
 
@@ -827,5 +916,23 @@ namespace Banter
 			actionManager ["QuitAction"].Activate ();
 		}
 		#endregion
+		
+		class GroupWindowMenuItem : ImageMenuItem
+		{
+			GroupWindow groupWindow;
+			
+			public GroupWindowMenuItem (GroupWindow groupWindow) :
+				base (groupWindow.SelectedGroup == null ?
+						Catalog.GetString ("Everyone") :
+						groupWindow.SelectedGroup.DisplayName)
+			{
+				this.groupWindow = groupWindow;
+			}
+			
+			public GroupWindow GroupWindow
+			{
+				get { return groupWindow; }
+			}
+		}
 	}
 }
